#include <stm32f10x.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_gpio.h>
#include <stm32f10x_spi.h>
#include "eeprom.h"
#include "spi.h"
#include "spi_dma.h"

#define EEPROM_SPI SPI2 //  spi.c    - SPI2
#define EEPROM_SPEED SPI_SLOW

#define PAGESIZE 16
//   ,       
//      .
#define MAX_NO_WRITES(x) (PAGESIZE - (x % PAGESIZE))

void eepromInit(void){

	 GPIO_InitTypeDef GPIO_InitStructure;

	 //slave select
	 RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC , ENABLE); //    EEPROM_PORT
	 GPIO_StructInit(&GPIO_InitStructure);
	 GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	 GPIO_InitStructure.GPIO_Pin =  EEPROM_CS;
	 GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;//GPIO_Speed_50MHz;
	 GPIO_Init(EEPROM_PORT, &GPIO_InitStructure);

	 GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1); //  
	 
	 /*Initialise SPI*/
	 spiInit(EEPROM_SPI);

	 spi_dma_Init();
}

uint8_t eepromReadStatus(void){
	uint8_t cmd[] = {cmdRDSR, 0xff};
	uint8_t res[2];
	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 0);
	asm("nop");
	asm("nop");
	spiReadWrite(EEPROM_SPI, res, cmd, 2, EEPROM_SPEED);
	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1);
	asm("nop");
	asm("nop");
	return res[1];
}

void eepromWriteEnable(void){
	uint8_t cmd = cmdWREN;

	while (WIP(eepromReadStatus()));
	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 0);
	spiReadWrite(EEPROM_SPI, 0, &cmd, 1, EEPROM_SPEED);
	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1);

}

void eepromWriteDisable(void){
	uint8_t cmd = cmdWRDI;
	
	while (WIP(eepromReadStatus()));
	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 0);
	spiReadWrite(EEPROM_SPI, 0, &cmd, 1, EEPROM_SPEED);
	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1);
	
}

void eepromWriteStatus(uint8_t status){
	uint8_t cmd[] = {cmdWRSR, status};

	while (WIP(eepromReadStatus())); //?
	while ( !WEL(eepromReadStatus()) )
		eepromWriteEnable(); // be set Write Enable Latch (WEL)

	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 0);
	spiReadWrite(EEPROM_SPI, 0, cmd, 2, EEPROM_SPEED);
	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1);

}

int eepromWrite(uint8_t *buf, uint8_t cnt, uint16_t offset){

	uint8_t cmd = cmdWRITE, rs;
	uint16_t factn, tail;
	uint16_t maxn = MAX_NO_WRITES(offset);

	while (WIP(eepromReadStatus()));
	while ( !WEL(eepromReadStatus()) )
		eepromWriteEnable(); // be set Write Enable Latch (WEL)

		factn = ( maxn > cnt) ?  cnt: maxn;
		//         
		GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 0);
		spiReadWrite(EEPROM_SPI, 0, &cmd, 1, EEPROM_SPEED);
		spiReadWrite16(EEPROM_SPI, 0, &offset, 1, EEPROM_SPEED);
		// DMA.    4 ,    DMA
		if ( factn > 4 ){
			xchng_datablock(EEPROM_SPI, 0, buf, 0, factn);
		}else{
			spiReadWrite(EEPROM_SPI, 0, buf, factn, EEPROM_SPEED);
		}
		GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1);

		offset += factn;

	//rs = eepromReadStatus();

		tail = cnt - factn;

			while ( tail > 0 ){
				//     
				factn = ( tail > PAGESIZE) ?  PAGESIZE: tail;

				//while (WIP(eepromReadStatus()));

ep222:				rs = eepromReadStatus();
					if (WIP(rs)) // ?
						goto ep222;

				 //  ,   WEL , 
				//     (,   )

				//  WEL,    
				while ( !WEL(eepromReadStatus()) ){
					eepromWriteEnable(); // will be set Write Enable Latch (WEL)
//
				}
				//rs = eepromReadStatus();

				GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 0);
				spiReadWrite(EEPROM_SPI, 0, &cmd, 1, EEPROM_SPEED);
				spiReadWrite16(EEPROM_SPI, 0, &offset, 1, EEPROM_SPEED);
				// DMA.    4 ,    DMA
				if ( factn > 4 ){
					xchng_datablock(EEPROM_SPI, 0, buf+maxn, 0, factn);
				}else{
					spiReadWrite(EEPROM_SPI, 0, buf+maxn, factn, EEPROM_SPEED);
				}

				GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1); // 

				offset += factn;
				maxn += factn;
				tail -= factn;
			}

	while (WIP(eepromReadStatus()));
	if ( !WEL(eepromReadStatus()) )
		return 0; //     WEL .
	else {
		eepromWriteDisable();
		return 1;
	}
}

int eepromRead(uint8_t *buf, uint8_t cnt, uint16_t offset){
	uint8_t cmd = cmdREAD;
	
	while (WIP(eepromReadStatus()));

	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 0);
	spiReadWrite(EEPROM_SPI, 0, &cmd, 1, EEPROM_SPEED);
	spiReadWrite16(EEPROM_SPI, 0, &offset, 1, EEPROM_SPEED);
	// DMA.    4 ,    DMA
	if ( cnt > 4 ){
		xchng_datablock(EEPROM_SPI, 0, 0, buf, cnt);
	}else{
		spiReadWrite(EEPROM_SPI, buf, 0, cnt, EEPROM_SPEED);
	}

	GPIO_WriteBit(EEPROM_PORT, EEPROM_CS, 1);
	
	return 0;
}
