/* eeprom.h */

#ifndef _EEPROM_
#define _EEPROM_

#include <stm32f10x.h>
#include <stm32f10x_gpio.h>

//    ChipSelect
#define EEPROM_PORT GPIOC
#define EEPROM_CS GPIO_Pin_10

#define WIP(x) ((x) & 1) //  "   ."
#define WEL(x) ((x) & 2) //  "   "

//
enum eepromCMD {
	cmdREAD = 0x03,  //   
	cmdWRITE = 0x02, //   
	cmdWREN = 0x06,  //  
	cmdWRDI = 0x04,  //  
	cmdRDSR = 0x05,  //  
	cmdWRSR = 0x01   //   
};

// .
//  SPI2 c   ChipSelect,    DMA1
void eepromInit(void);

//      .  :
// 0 - WIP  //    
// 1 - WEL  // ""  .
// 2 - BPO  //     
// 3 - BP1  //     
// 4 - -
// 5 - -
// 6 - -
// 7 - WPEN //        

//WIP, WEL -  
//BPO, BP1, WPEN -   , .

//    (      
// )    WEL   .  
//   WREN.
//          
// : WRDI, WRSR, WRITE.
// (      eepromWriteStatus()  eepromWrite()
//    -    )
void eepromWriteEnable(void);

// ,   eepromWriteEnable(),   ().
void eepromWriteDisable(void);

//  
// :   
uint8_t eepromReadStatus(void);

//  WRSR.
//       BP0  BP1   
//  ""   .
//"":
// _BP1__BP0
// 1. 0  0  -  
// 2. 0  1  -   1/4  ( 0600h - 07FFh)
// 3. 1  0  -   1/2  ( 0400h - 07FFh)
// 4. 1  1  -    ( 0000h - 07FFh)
void eepromWriteStatus(uint8_t status);

//    .
// buf -   
// cnt -  
// offset -     
int eepromWrite(uint8_t *buf, uint8_t cnt, uint16_t offset);

//    .
// buf -  
// cnt -  
// offset -     
int eepromRead(uint8_t *buf, uint8_t cnt, uint16_t offset);

//  dataSheet 25AA160/25LC160/25C160  
// TABLE 3-3: WRITE-PROTECT FUNCTIONALITY MATRIX
//     WPEN  WEL   WP  
//     .
// (    WP     "+".)
#endif
