#include <stm32f10x.h>
#include <stm32f10x_rcc.h>
//#include <stm32f10x_gpio.h>
#include <stm32f10x_spi.h>
#include <stm32f10x_dma.h>
#include "spi_dma.h"

static int dmaRcvBytes(SPI_TypeDef *SPIx, uint8_t half, void *rbuf , unsigned count);
static int dmaTrnsmtBytes(SPI_TypeDef *SPIx, uint8_t half, const void *tbuf , unsigned count);
static int dmaExchangeBytes(const const void *tbuf, void *rbuf, uint8_t half,  unsigned count);

void spi_dma_Init(void){

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
}

int xchng_datablock(SPI_TypeDef *SPIx, uint8_t half, const void *tbuf, void *rbuf, unsigned count){

	int i;

	if(!tbuf && rbuf) { //receive

		i = dmaRcvBytes(SPIx, half, rbuf, count);

	} else if (tbuf && !rbuf) { //transmit

		i = dmaTrnsmtBytes(SPIx, half, tbuf, count);

	} else if ( tbuf && rbuf ){ //exchange
		i = dmaExchangeBytes(tbuf, rbuf, half, count);
	}


	return i;						// 

}

static int dmaRcvBytes(SPI_TypeDef *SPIx, uint8_t half, void *rbuf , unsigned count){

DMA_InitTypeDef DMA_InitStructure;
DMA_Channel_TypeDef *rxChan, *txChan;
uint32_t DMA1_FLAG_TC;

uint16_t dummy[] = {0xffff};

if ( SPIx == SPI1 ){
	rxChan = DMA1_Channel2;
	txChan = DMA1_Channel3;
	DMA1_FLAG_TC = DMA1_FLAG_TC2;
} else if ( SPIx == SPI2 ){
	rxChan = DMA1_Channel4;
	txChan = DMA1_Channel5;
	DMA1_FLAG_TC = DMA1_FLAG_TC4;
}

DMA_DeInit(rxChan);
DMA_DeInit(txChan);
// Common to both channels
DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)(&(SPIx->DR));
if( !half ){
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
}else{
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;

	SPI_Cmd(SPIx, DISABLE);		// page 562 in RM0041
	SPI_DataSizeConfig(SPIx, SPI_DataSize_16b); //temporarily changes data size to 16b
	SPI_Cmd(SPIx, ENABLE);
}
DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
DMA_InitStructure.DMA_BufferSize = count;
DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
// Rx Channel
DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)rbuf;
DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
DMA_Init(rxChan, &DMA_InitStructure);
// Tx channel
DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) dummy;
DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
DMA_Init(txChan , &DMA_InitStructure);

// Enable channels
DMA_Cmd(rxChan, ENABLE);
DMA_Cmd(txChan, ENABLE);

// Enable SPI TX/RX request
SPI_I2S_DMACmd(SPIx, SPI_I2S_DMAReq_Rx | SPI_I2S_DMAReq_Tx, ENABLE);

// Wait for completion
while (DMA_GetFlagStatus(DMA1_FLAG_TC) == RESET);    //   

DMA1->IFCR = DMA1_FLAG_TC; // clear

// Disable channels
DMA_Cmd(rxChan, DISABLE);
DMA_Cmd(txChan, DISABLE);
SPI_I2S_DMACmd(SPIx, SPI_I2S_DMAReq_Rx | SPI_I2S_DMAReq_Tx, DISABLE);

if( half ){
	SPI_Cmd(SPIx, DISABLE);
	SPI_DataSizeConfig(SPIx, SPI_DataSize_8b); //changes data size back to 8b
	SPI_Cmd(SPIx, ENABLE);
}
return count;
}

static int dmaTrnsmtBytes(SPI_TypeDef *SPIx, uint8_t half, const void *tbuf , unsigned count){

DMA_InitTypeDef DMA_InitStructure;
DMA_Channel_TypeDef *txChan;
uint32_t DMA1_FLAG_TC;
uint16_t SR;

if ( SPIx == SPI1 ){
	txChan = DMA1_Channel3;
	DMA1_FLAG_TC = DMA1_FLAG_TC3;
} else if ( SPIx == SPI2 ){
	txChan = DMA1_Channel5;
	DMA1_FLAG_TC = DMA1_FLAG_TC5;
}

DMA_DeInit(txChan);

DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)(&(SPIx->DR));
if( !half ){
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
}else{
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;

	SPI_Cmd(SPIx, DISABLE);		// page 562 in RM0041
	SPI_DataSizeConfig(SPIx, SPI_DataSize_16b); //temporarily changes data size to 16b
	SPI_Cmd(SPIx, ENABLE);
}
DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
DMA_InitStructure.DMA_BufferSize = count;
DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

//
DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)tbuf;
DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
DMA_Init(txChan, &DMA_InitStructure);

// Enable channel
DMA_Cmd(txChan, ENABLE);

// Enable SPI TX request
SPI_I2S_DMACmd(SPIx, SPI_I2S_DMAReq_Tx, ENABLE);

// Wait for completion
while (DMA_GetFlagStatus(DMA1_FLAG_TC) == RESET)    //   
	;
DMA1->IFCR = DMA1_FLAG_TC; // clear

while (SPI_I2S_GetFlagStatus(SPIx, SPI_I2S_FLAG_TXE) == RESET );
while (SPI_I2S_GetFlagStatus(SPIx, SPI_I2S_FLAG_BSY) != RESET );

//       -  -   RXNE  OVR .
//     DR.
SR = SPIx->DR;

// Disable channels
DMA_Cmd(txChan, DISABLE);
SPI_I2S_DMACmd(SPIx, SPI_I2S_DMAReq_Tx, DISABLE);

if( half ){
	SPI_Cmd(SPIx, DISABLE);
	SPI_DataSizeConfig(SPIx, SPI_DataSize_8b); //changes data size back to 8b
	SPI_Cmd(SPIx, ENABLE);
}
return count;
}

//  rbuf  tbuf
static int dmaExchangeBytes(const void *tbuf, void *rbuf, uint8_t half,  unsigned count){

DMA_InitTypeDef DMA_InitStructure;
DMA_Channel_TypeDef *Chan;

Chan = DMA1_Channel2;   //  

DMA_DeInit(Chan);

DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)rbuf;
if( !half ){
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
}else{
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
}
DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Enable;
DMA_InitStructure.DMA_BufferSize = count;
DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
DMA_InitStructure.DMA_M2M = DMA_M2M_Enable;

DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)tbuf;
DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
DMA_Init(Chan, &DMA_InitStructure);

// Enable channels
DMA_Cmd(Chan, ENABLE);

// Wait for completion
while (DMA_GetFlagStatus(DMA1_FLAG_TC2) == RESET);    //   

DMA1->IFCR = DMA1_FLAG_TC2; // clear

// Disable channels
DMA_Cmd(Chan, DISABLE);

return count;
}
